"""
通达信 小达
https://wenda.tdx.com.cn/
"""
import math
import re

import pandas as pd
from loguru import logger
from playwright.async_api import Page

from mcp_query_table.enums import QueryType

# 查询结果
_PAGE1_ = 'https://wenda.tdx.com.cn/TQL?Entry=NLPSE.NLPQuery'
# 代码数量
_PAGE2_ = 'https://wenda.tdx.com.cn/TQL?Entry=JNLPSE.getAllCode'

_queryType_ = {
    QueryType.CNStock: 'AG',
    QueryType.Fund: 'JJ',
    QueryType.Index: 'ZS',
    QueryType.Info: 'ZX',
    QueryType.Board: 'ZS',  # 板块也走指数
}


def convert_type(type):
    if type == '':
        return str
    if type == '0|0|0':
        return str
    if type == '2|0|0':
        return float
    if type == '0|9|1':
        return float
    if type == '1|9|1':
        return float
    if type == '2|9|1':
        return float
    return type


class Pagination:
    def __init__(self):
        self.datas = {}
        self.last_count = 1
        self.limit = 100
        self.row_count = 1024
        self.dtypes = []
        self.columns = []

    def reset(self):
        self.datas = {}

    def update_row_count(self, row_count):
        self.row_count = row_count

    def update_last_count(self, limit, last_count, columns, dtypes, datas):
        self.limit = limit
        self.last_count = last_count
        self.columns = columns
        self.dtypes = dtypes
        self.datas[last_count] = datas

    def has_next(self, max_page):
        page = math.ceil(self.last_count / self.limit)
        c1 = self.last_count < self.row_count
        c2 = page < max_page
        return c1 & c2

    def current(self):
        return self.last_count

    def get_list(self):
        datas = []
        for k, v in self.datas.items():
            datas.extend(v)
        return datas

    def get_dataframe(self, rename: bool):
        dtypes = [convert_type(x) for x in self.dtypes]
        df = pd.DataFrame(self.get_list(), columns=self.columns)
        for i, v in enumerate(dtypes):
            k = self.columns[i]
            if k == 'POS':
                df[k] = df[k].astype(int)
                continue
            if isinstance(v, str):
                logger.info("未识别的数据类型 {}:{}", k, v)
                continue
            try:
                df[k] = df[k].astype(v)
            except ValueError:
                logger.info("转换失败 {}:{}", k, v)
        return df


P = Pagination()


def NLPQuery(json_data):
    limit = json_data[0][2]
    last_count = int(json_data[0][4])
    columns = json_data[1]
    dtypes = json_data[2]
    datas = json_data[3:]

    return limit, last_count, columns, dtypes, datas


def getAllCode(json_data):
    row_count = json_data[0][2]

    return row_count


async def on_response1(response):
    if response.url.startswith(_PAGE1_):
        P.update_last_count(*NLPQuery(await response.json()))


async def on_response2(response):
    if response.url.startswith(_PAGE2_):
        P.update_row_count(getAllCode(await response.json()))


async def query(page: Page,
                message: str = "收盘价>100元",
                type_: QueryType = 'AG',
                max_page: int = 5,
                rename: bool = False) -> pd.DataFrame:
    queryType = _queryType_.get(type_, None)
    assert queryType is not None, f"不支持的类型:{type_}"

    await page.route(re.compile(r'.*\.(?:jpg|jpeg|png|gif|webp)(?:$|\?)'), lambda route: route.abort())
    page.on("response", on_response2)

    P.reset()
    async with page.expect_response(lambda response: response.url.startswith(_PAGE1_)) as response_info:
        await page.goto(f"https://wenda.tdx.com.cn/site/wenda/stock_index.html?message={message}&queryType={queryType}",
                        wait_until="load")
    await on_response1(await response_info.value)

    while P.has_next(max_page):
        logger.info("当前序号为:{}, 点击`下一页`", P.current())
        async with page.expect_response(lambda response: response.url.startswith(_PAGE1_)) as response_info:
            await page.get_by_role("button", name="下一页").click()
        await on_response1(await response_info.value)

    return P.get_dataframe(rename)



"""
360 纳米搜索
"""
import json

from loguru import logger
from playwright.async_api import Page

import mcp_query_table
from mcp_query_table.utils import is_image, GlobalVars

_PAGE0_ = "https://www.n.cn"
_PAGE1_ = "https://www.n.cn/search"
_PAGE2_ = "https://www.n.cn/api/common/chat/v2"  # 对话
_PAGE3_ = "https://www.n.cn/api/image/upload"  # 上传图片

G = GlobalVars()


def read_event_stream(text):
    text1 = []
    text2 = []
    for event in text.split('\n\n'):
        if "event: 102" in event:
            if 'data: {"type":"reasoning_text"' in event:
                lines = event.split('\n')
                for line in lines:
                    if line.startswith('data: '):
                        t = line[6:]
                        t = json.loads(t)['message']
                        text1.append(t)
        if "event: 200" in event:
            lines = event.split('\n')
            for line in lines:
                if line.startswith('data: '):
                    t = line[6:]
                    if t == '':
                        text2.append('\n')
                    elif t == ' ':
                        text2.append('\n')
                    else:
                        text2.append(t)

    text2 = ''.join(text2)
    if len(text1) == 0:
        return text2
    else:
        text1 = ''.join(text1)
        return f"<thinking>{text1}</thinking>\n\n{text2}"


async def on_response(response):
    if response.url == _PAGE2_:
        # print("on_response", response.url)
        text = await response.text()
        G.set_text(read_event_stream(text))


async def chat(page: Page,
               prompt: str,
               create: bool,
               files: list[str],
               ) -> str:
    """

    Parameters
    ----------
    page : playwright.async_api.Page
        页面
    prompt : str
        问题
    create : bool
        是否创建新的对话
    files : list[str] | None
        上传的文件列表。目前仅支持上传图片

    Returns
    -------
    str
        回答
    """
    logger.warning("纳米搜索。不登录可以使用。但无头模式要指定`user_data_dir`才能正常工作")

    if not create:
        if not page.url.startswith(_PAGE1_):
            create = True
        if len(files) > 0:
            create = True

    for file in files:
        assert is_image(file), f"仅支持上传图片，{file}不是图片"

    if create:
        name = "输入任何问题"

        await page.goto(_PAGE0_)
        if len(files) > 0:
            # 只能在新会话中上传文件
            async with page.expect_response(_PAGE3_, timeout=mcp_query_table.TIMEOUT_60) as response_info:
                await page.locator("input[type=\"file\"]").set_input_files(files)
    else:
        name = "提出后续问题，Enter发送，Shift+Enter 换行"

    async with page.expect_response(_PAGE2_, timeout=mcp_query_table.TIMEOUT) as response_info:
        textbox = page.get_by_role("textbox", name=name)
        await textbox.fill(prompt)
        await textbox.press("Enter")
        # await page.screenshot(path="n.png")
    await on_response(await response_info.value)

    return G.get_text()



"""
百度AI搜索

限制了输入长度为5000，很多时候会被截断，导致MCP无法正常工作
"""
import json

from playwright.async_api import Page

import mcp_query_table
from mcp_query_table.utils import split_images, GlobalVars

_PAGE0_ = "https://chat.baidu.com/search"
_PAGE1_ = "https://chat.baidu.com/aichat/api/conversation"
_PAGE2_ = "https://chat.baidu.com/aichat/api/file/upload"

G = GlobalVars()


def read_event_stream(text):
    text1 = []
    text2 = []
    for event in text.split('\n\n'):
        if '"component":"thinkingSteps"' in event:
            if '"reasoningContent":' not in event:
                continue
            lines = event.split('\n')
            for line in lines:
                if line.startswith('data:'):
                    t = line[5:]
                    t = json.loads(t)['data']['message']['content']['generator']['data']['reasoningContent']
                    text1.append(t)
        if '"component":"markdown-yiyan"' in event:
            lines = event.split('\n')
            for line in lines:
                if line.startswith('data:'):
                    t = line[5:]
                    t = json.loads(t)['data']['message']['content']['generator']['data']['value']
                    text2.append(t)

    text2 = ''.join(text2)
    if len(text1) == 0:
        return text2
    else:
        text1 = ''.join(text1)
        return f"<thinking>{text1}</thinking>\n\n{text2}"


async def on_response(response):
    if response.url.startswith(_PAGE1_):
        # print("on_response", response.url)
        text = await response.text()
        G.set_text(read_event_stream(text))


async def on_route(route):
    # 避免出现 Protocol error (Network.getResponseBody): No data found for resource with given identifier
    # print("on_route", route.request.url)
    if route.request.url == _PAGE1_:
        # TODO 为何只要转发一下就没事了？
        response = await route.fetch(timeout=mcp_query_table.TIMEOUT)
        await route.fulfill(response=response)
    else:
        await route.continue_()


async def chat(page: Page,
               prompt: str,
               create: bool,
               files: list[str],
               ) -> str:
    async def on_file_chooser(file_chooser):
        # 文件选择对话框
        await file_chooser.set_files(files)

    if not page.url.startswith(_PAGE0_):
        create = True

    if create:
        await page.goto(_PAGE0_)

    # 文件上传
    if len(files) > 0:
        imgs, docs = split_images(files)
        assert len(imgs) == 0 or len(docs) == 0, "不能同时包含图片和文档"

        page.on("filechooser", on_file_chooser)
        async with page.expect_response(f"{_PAGE2_}*", timeout=mcp_query_table.TIMEOUT_60) as response_info:
            if len(imgs) > 0:
                await page.locator(".cs-input-upload-icon").last.click()
            else:
                await page.locator(".cs-input-upload-icon").first.click()
        page.remove_listener("filechooser", on_file_chooser)

    # 提交问题
    await page.route(_PAGE1_, on_route)
    async with page.expect_response(_PAGE1_, timeout=mcp_query_table.TIMEOUT) as response_info:
        await page.locator("#chat-input-box").fill(prompt)
        await page.locator("#chat-input-box").press("Enter")
    await on_response(await response_info.value)

    return G.get_text()



"""
腾讯元宝
"""
import json
import re

from loguru import logger
from playwright.async_api import Page

import mcp_query_table
from mcp_query_table.utils import split_images, GlobalVars

_PAGE0_ = "https://yuanbao.tencent.com/"
_PAGE1_ = "https://yuanbao.tencent.com/api/chat"
_PAGE2_ = "https://yuanbao.tencent.com/api/resource/genUploadInfo"

G = GlobalVars()


def read_event_stream(text):
    text1 = []
    text2 = []
    for event in text.split('\n\n'):
        if 'data: {"type":"think"' in event:
            lines = event.split('\n')
            for line in lines:
                if line.startswith('data: '):
                    t = line[6:]
                    t = json.loads(t)['content']
                    text1.append(t)
        if 'data: {"type":"text"' in event:
            lines = event.split('\n')
            for line in lines:
                if line.startswith('data: '):
                    t = line[6:]
                    t = json.loads(t).get('msg', "")
                    text2.append(t)

    text2 = ''.join(text2)
    if len(text1) == 0:
        return text2
    else:
        text1 = ''.join(text1)
        return f"<thinking>{text1}</thinking>\n\n{text2}"


async def on_response(response):
    if response.url.startswith(_PAGE1_):
        # print("on_response", response.url)
        text = await response.text()
        G.set_text(read_event_stream(text))


async def on_route(route):
    # print("on_route", route.request.url)
    if route.request.url.startswith(_PAGE1_):
        # TODO 这里会导致数据全部加载，逻辑变了，所以界面可能混乱
        response = await route.fetch(timeout=mcp_query_table.TIMEOUT)
        await route.fulfill(
            # 强行加utf-8，否则编码搞不定
            content_type="text/event-stream; charset=utf-8",
            response=response,
        )
    else:
        await route.continue_()


async def chat(page: Page,
               prompt: str,
               create: bool,
               files: list[str]
               ) -> str:
    logger.info("腾讯元宝。登录才可以使用。无头模式时要指定`user_data_dir`才能正常工作")

    if not page.url.startswith(_PAGE0_):
        create = True

    if create:
        await page.goto(_PAGE0_)

    if len(files) > 0:
        imgs, docs = split_images(files)
        assert len(imgs) == 0 or len(docs) == 0, "不能同时包含图片和文档"

        # 点击上传文件按钮，才会出现上传文件的input
        await page.get_by_role("button").filter(has_text=re.compile(r"^$")).last.click()

        # 上传文件
        async with page.expect_response(_PAGE2_, timeout=mcp_query_table.TIMEOUT_60) as response_info:
            if len(imgs) > 0:
                await page.locator("input[type=\"file\"]").nth(-2).set_input_files(files)
            else:
                await page.locator("input[type=\"file\"]").last.set_input_files(files)

    # 提问
    await page.route(f"{_PAGE1_}/*", on_route)
    async with page.expect_response(f"{_PAGE1_}/*", timeout=mcp_query_table.TIMEOUT) as response_info:
        textbox = page.locator(".ql-editor")
        await textbox.fill(prompt)
        await textbox.press("Enter")
    await on_response(await response_info.value)

    return G.get_text()



import getpass

from mcp_query_table.server import serve


def main():
    import argparse

    parser = argparse.ArgumentParser(
        description="query table from website",
    )

    parser.add_argument("--format", type=str, help="输出格式",
                        default='markdown', choices=['markdown', 'csv', 'json'])
    parser.add_argument("--endpoint", type=str, help="浏览器CDP地址/WS地址",
                        nargs="?", default=r'http://127.0.0.1:9222')
    parser.add_argument("--executable_path", type=str, help="浏览器路径",
                        nargs="?", default=r'C:\Program Files\Google\Chrome\Application\chrome.exe')
    parser.add_argument("--user_data_dir", type=str, help="浏览器用户数据目录",
                        nargs="?", default=rf'C:\Users\{getpass.getuser()}\AppData\Local\Google\Chrome\User Data')
    parser.add_argument("--transport", type=str, help="传输类型",
                        default='stdio', choices=['stdio', 'sse', 'streamable-http'])
    parser.add_argument("--host", type=str, help="MCP服务端绑定地址",
                        default='0.0.0.0')
    parser.add_argument("--port", type=int, help="MCP服务端绑定端口",
                        default='8000')
    args = parser.parse_args()
    serve(args.format, args.endpoint,
          args.executable_path, args.user_data_dir,
          args.transport, args.host, args.port)


if __name__ == "__main__":
    main()



import getpass
import subprocess
import sys
import time
from pathlib import Path
from typing import Optional
from urllib.parse import urlparse, quote

import pandas as pd
from loguru import logger
from playwright.async_api import async_playwright, Playwright, Page
from playwright_stealth import Stealth

from mcp_query_table.enums import QueryType, Site, Provider


def create_detached_process(command):
    # 设置通用参数
    kwargs = {}

    if sys.platform == 'win32':
        kwargs.update({
            # 在PyCharm中运行还是会出现新建进程被关闭
            'creationflags': subprocess.DETACHED_PROCESS | subprocess.CREATE_NEW_PROCESS_GROUP
        })
    else:
        # Unix-like 系统（Linux, macOS）特定设置
        kwargs.update({
            'start_new_session': True  # 创建新的会话
        })
    logger.info(f"Popen: {command}")
    return subprocess.Popen(command, **kwargs)


def is_local_url(url: str) -> bool:
    """判断url是否是本地地址"""
    for local in ('localhost', '127.0.0.1'):
        if local in url.lower():
            return True
    return False


def is_cdp_url(url: str) -> bool:
    """判断url是否是CDP地址"""
    if url.startswith('ws://') or url.startswith('wss://'):
        return False
    return True


def get_executable_path(executable_path) -> Optional[str]:
    """获取浏览器可执行文件路径"""
    browsers = {
        "default": executable_path,
        "chrome.exe": r"C:\Program Files\Google\Chrome\Application\chrome.exe",
        "msedge.exe": r"C:\Program Files (x86)\Microsoft\Edge\Application\msedge.exe",
    }
    for k, v in browsers.items():
        if v is None:
            continue
        if Path(v).exists():
            return v
    return None


def get_user_data_dir(user_data_dir) -> Optional[str]:
    """获取浏览器可用户目录"""
    browsers = {
        "default": user_data_dir,
        "chrome.exe": rf'C:\Users\{getpass.getuser()}\AppData\Local\Google\Chrome\User Data',  # 使用默认配置文件时无法创建CDP
        "msedge.exe": rf"C:\Users\{getpass.getuser()}\AppData\Local\Microsoft\Edge\User Data",
    }
    for k, v in browsers.items():
        if v is None:
            continue
        if Path(v).exists():
            return v
    return None


class BrowserManager:
    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.cleanup()

    def __init__(self,
                 endpoint: Optional[str],
                 executable_path: Optional[str] = None,
                 devtools: bool = False,
                 headless: bool = True,
                 user_data_dir: Optional[str] = None):
        """

        Parameters
        ----------
        endpoint:str or None
            浏览器CDP地址/WS地址。
            如果为None，则直接启动浏览器实例。可用无头模式。建议指定用户数据目录，否则可能无法使用某些需要登录的网站
        executable_path:str
            浏览器可执行文件路径。推荐使用chrome，因为Microsoft Edge必须在任务管理器中完全退出才能启动调试端口
        devtools:bool
            是否显示开发者工具
        headless:bool
            是否无头模式启动浏览器
        user_data_dir:str
            浏览器用户数据目录。无头模式。强烈建议指定用户数据目录，否则可能无法使用某些需要登录的网站

        """
        if devtools:
            headless = False

        self.endpoint = endpoint
        self.executable_path = executable_path
        self.devtools = devtools
        self.headless = headless
        self.user_data_dir = user_data_dir

        self.playwright: Optional[Playwright] = None
        self.browser = None
        self.context = None
        # 空闲page池
        self.pages = []

    async def cleanup(self):
        if self.browser:
            await self.browser.close()
        if self.playwright:
            await self.playwright.stop()

    async def _connect_to_local(self) -> None:
        """连接本地浏览器"""
        port = urlparse(self.endpoint).port
        executable_path = get_executable_path(self.executable_path)
        name = Path(executable_path).name
        command = [executable_path, f'--remote-debugging-port={port}', '--start-maximized']
        if self.devtools:
            command.append('--auto-open-devtools-for-tabs')
        if self.user_data_dir:
            command.append(f'--user-data-dir={self.user_data_dir}')
        else:
            logger.warning('Chrome必须另行指定`--user-data-dir`才能创建CDP连接')

        for i in range(2):
            try:
                self.browser = await self.playwright.chromium.connect_over_cdp(self.endpoint,
                                                                               timeout=10000, slow_mo=1000)
                break
            except:
                if i == 0:
                    create_detached_process(command)
                    time.sleep(5)
                    continue
                if i == 1:
                    raise ConnectionError(
                        f"已提前打开了浏览器，但未开启远程调试端口？请关闭浏览器全部进程后重试 `taskkill /f /im {name}`")

    async def _connect_to_remote(self) -> None:
        """连接远程浏览器"""
        try:
            if is_cdp_url(self.endpoint):
                self.browser = await self.playwright.chromium.connect_over_cdp(self.endpoint,
                                                                               timeout=10000, slow_mo=1000)
            else:
                self.browser = await self.playwright.chromium.connect(self.endpoint,
                                                                      timeout=10000, slow_mo=1000)
        except:
            raise ConnectionError(f"连接远程浏览器失败，请检查CDP/WS地址和端口是否正确。{self.endpoint}")

    async def _connect_to_launch(self) -> None:
        logger.info("executable_path={}", self.executable_path)
        if self.user_data_dir:
            logger.info("user_data_dir={}", self.user_data_dir)
            try:
                self.context = await self.playwright.chromium.launch_persistent_context(
                    user_data_dir=self.user_data_dir,
                    executable_path=self.executable_path,
                    headless=self.headless,
                    devtools=self.devtools,
                    timeout=10000, slow_mo=1000)
            except:
                raise ConnectionError(f"launch失败，可能已经有浏览器已经打开了数据目录。{self.user_data_dir}")
        else:
            logger.warning("未指定浏览器用户数据目录，部分需要的网站可能无法使用")
            self.browser = await self.playwright.chromium.launch(
                executable_path=self.executable_path,
                headless=self.headless,
                devtools=self.devtools)

    async def _launch(self) -> None:
        """启动浏览器，并连接CDP协议

        References
        ----------
        https://blog.csdn.net/qq_30576521/article/details/142370538

        """
        self.playwright = await async_playwright().start()
        if self.endpoint is None:
            await self._connect_to_launch()
        elif is_local_url(self.endpoint) and is_cdp_url(self.endpoint):
            await self._connect_to_local()
        else:
            await self._connect_to_remote()

        if self.browser is None:
            pass
        elif len(self.browser.contexts) == 0:
            self.context = await self.browser.new_context()
        else:
            self.context = self.browser.contexts[0]
        # 爱问财，无头模式，需要使用 stealth 插件
        await Stealth().apply_stealth_async(self.context)

        # 复用打开的page
        for page in self.context.pages:
            # 防止开发者工具被使用
            if page.url.startswith("devtools://"):
                continue
            # 防止chrome扩展被使用
            if page.url.startswith("chrome-extension://"):
                continue
            # 防止edge扩展被使用
            if page.url.startswith("extension://"):
                continue
            self.pages.append(page)

    async def get_page(self) -> Page:
        """获取可用Page。无空闲标签时会打开新标签"""
        if self.context is None:
            await self._launch()

        # 反复取第一个tab
        while len(self.pages) > 0:
            page = self.pages.pop()
            if page.is_closed():
                continue
            return page

        # 不够，新建一个
        return await self.context.new_page()

    def release_page(self, page) -> None:
        """用完的Page释放到池中。如果用完不放回，get_page会一直打开新标签"""
        if page.is_closed():
            return
        # 放回
        self.pages.append(page)


async def query(
        page: Page,
        query_input: str = "收盘价>100元",
        query_type: QueryType = QueryType.CNStock,
        max_page: int = 5,
        rename: bool = False,
        site: Site = Site.THS,
) -> pd.DataFrame:
    """查询表格

    Parameters
    ----------
    page : playwright.sync_api.Page
        页面
    query_input : str, optional
        查询条件, by default "收盘价>100元"
    query_type : QueryType, optional
        查询类型, by default QueryType.astock
    max_page : int, optional
        最大页数, by default 5
    rename: bool
        是否重命名列名, by default False
    site : Site, optional
        站点, by default Site.iwencai

    Returns
    -------
    pd.DataFrame
        查询结果

    """
    query_input = quote(query_input.strip(), safe='')

    if site == Site.EastMoney:
        from mcp_query_table.sites.eastmoney import query
        return await query(page, query_input, query_type, max_page, rename)
    if site == Site.THS:
        from mcp_query_table.sites.iwencai import query
        return await query(page, query_input, query_type, max_page, rename)
    if site == Site.TDX:
        from mcp_query_table.sites.tdx import query
        return await query(page, query_input, query_type, max_page, rename)

    raise ValueError(f"未支持的站点:{site}")


async def chat(
        page: Page,
        prompt: str = "9.9大还是9.11大？",
        create: bool = False,
        files: list[str] | None = None,
        provider: Provider = Provider.Nami) -> str:
    """大语言对话

    Parameters
    ----------
    page : playwright.sync_api.Page
        页面
    prompt : str, optional
        对话内容, by default "9.9大还是9.11大？"
    create : bool, optional
        是否创建新对话, by default False
    files : list[str] | None, optional
        上传的文件列表。不同网站支持程度不同
    provider : Provider, optional
        提供商, by default Provider.N

    Returns
    -------
    str
        对话结果

    """
    # 空列表转None
    if files is None:
        files = []

    if provider == Provider.Nami:
        from mcp_query_table.providers.n import chat
        return await chat(page, prompt, create, files)
    if provider == Provider.YuanBao:
        from mcp_query_table.providers.yuanbao import chat
        return await chat(page, prompt, create, files)
    if provider == Provider.BaiDu:
        from mcp_query_table.providers.baidu import chat
        return await chat(page, prompt, create, files)

    raise ValueError(f"未支持的提供商:{provider}")



from typing import Annotated, List

import fastmcp
from loguru import logger
from pydantic import Field

from mcp_query_table import QueryType, Site, query as qt_query, chat as qt_chat
from mcp_query_table.enums import Provider
from mcp_query_table.tool import BrowserManager


class QueryServer:
    def __init__(self) -> None:
        self.format: str = "markdown"
        self.browser = None

    def start(self, format, endpoint, executable_path, user_data_dir):
        self.format: str = format
        self.browser = BrowserManager(endpoint=endpoint,
                                      executable_path=executable_path,
                                      user_data_dir=user_data_dir,
                                      devtools=False,
                                      headless=True)

    async def query(self, query_input: str, query_type: QueryType, max_page: int, rename: bool, site: Site):
        page = await self.browser.get_page()
        df = await qt_query(page, query_input, query_type, max_page, rename, site)
        self.browser.release_page(page)

        if self.format == 'csv':
            return df.to_csv()
        if self.format == 'markdown':
            return df.to_markdown()
        if self.format == 'json':
            return df.to_json(force_ascii=False, indent=2)

    async def chat(self, prompt: str, create: bool, files: List[str], provider: Provider):
        page = await self.browser.get_page()
        txt = await qt_chat(page, prompt, create, files, provider)
        self.browser.release_page(page)
        return txt


mcp = fastmcp.FastMCP("query_table_mcp")
qsv = QueryServer()


@mcp.tool(description="查询金融表格数据")
async def query(
        query_input: Annotated[
            str, Field(description="查询条件。支持复杂查询，如：`2024年涨幅最大的100只股票按市值排名`")],
        query_type: Annotated[QueryType, Field(default=QueryType.CNStock,
                                               description="查询类型。支持`A股`、`指数`、`基金`、`港股`、`美股`等")],
        max_page: Annotated[int, Field(default=1, ge=1, le=10, description="最大页数。只查第一页即可")],
        rename: Annotated[bool, Field(default=False, description="是否重命名列名")],
        site: Annotated[Site, Field(default=Site.THS, description="站点。支持`东方财富`、`通达信`、`同花顺`")]
) -> str:
    return await qsv.query(query_input, query_type, max_page, rename, site)


# chat功能不通过mcp暴露，因为在Cline等客户端中本就有LLM功能，反而导致返回的数据没有正确提交
# @mcp.tool(description="大语言模型对话")
async def chat(
        prompt: Annotated[str, Field(description="提示词。如：`9.9大还是9.11大？`")],
        create: Annotated[bool, Field(default=False, description="是否创建新对话")],
        files: Annotated[List[str], Field(default=None, description="上传的文件列表。不同网站支持程度不同")],
        provider: Annotated[
            Provider, Field(default=Provider.Nami, description="提供商。支持`纳米搜索`、`腾讯元宝`、`百度AI搜索`")]
) -> str:
    return await qsv.chat(prompt, create, files, provider)


def serve(format, endpoint, executable_path, user_data_dir, transport, host, port):
    qsv.start(format, endpoint, executable_path, user_data_dir)
    logger.info(f"{endpoint=}")
    logger.info(f"{executable_path=}")
    logger.info(f"{user_data_dir=}")

    if transport == "stdio":
        logger.info(f"{transport=},{format=}")
        mcp.run(transport=transport)
    else:
        logger.info(f"{transport=},{format=},{host=},{port=}")
        mcp.run(transport=transport, host=host, port=port)



"""
同花顺问财
https://www.iwencai.com/

1. 一定要保证浏览器宽度>768，防止界面变成适应手机

"""
import re

import pandas as pd
from loguru import logger
from playwright.async_api import Page

from mcp_query_table.enums import QueryType

# 初次查询页面
_PAGE1_ = 'https://www.iwencai.com/customized/chart/get-robot-data'
# 翻页
_PAGE2_ = 'https://www.iwencai.com/gateway/urp/v7/landing/getDataList'

_querytype_ = {
    QueryType.CNStock: 'stock',
    QueryType.Index: 'zhishu',
    QueryType.Fund: 'fund',
    QueryType.HKStock: 'hkstock',
    QueryType.USStock: 'usstock',
    '新三板': 'threeboard',
    QueryType.ConBond: 'conbond',
    '保险': 'insurance',
    '期货': 'futures',
    '理财': 'lccp',
    '外汇': 'foreign_exchange',
    '宏观': 'macro',
    #
    QueryType.ETF: 'fund',  # 查ETF定位到基金
}


def convert_type(type):
    if type == 'LONG':
        return int
    if type == 'DOUBLE':
        return float
    if type == 'STR':
        return str
    if type == 'INT':  # TODO 好像未出现过
        return int
    return type


class Pagination:
    def __init__(self):
        self.datas = {}
        self.limit = 100
        self.page = 1
        self.row_count = 1024
        self.columns = []

    def reset(self):
        self.datas = {}

    def update(self, datas, columns, page, limit, row_count):
        self.datas[page] = datas
        self.columns = columns
        self.limit = limit
        self.page = page
        self.row_count = row_count

    def has_next(self, max_page):
        c1 = self.page * self.limit < self.row_count
        c2 = self.page < max_page
        return c1 & c2

    def current(self):
        return self.page

    def get_list(self):
        datas = []
        for k, v in self.datas.items():
            datas.extend(v)
        return datas

    def get_dataframe(self, rename: bool):
        columns = {x['key']: x['index_name'] for x in self.columns}
        dtypes = {x['key']: convert_type(x['type']) for x in self.columns}

        df = pd.DataFrame(self.get_list())
        for k, v in dtypes.items():
            if isinstance(v, str):
                logger.info("未识别的数据类型 {}:{}", k, v)
                continue
            try:
                df[k] = df[k].astype(v)
            except ValueError:
                logger.info("转换失败 {}:{}", k, v)

        if rename:
            return df.rename(columns=columns)
        else:
            return df


P = Pagination()


def get_robot_data(json_data):
    """
json_data['data']['answer'][0]['txt'][0]['content']['components'][0]['data']['datas']
json_data['data']['answer'][0]['txt'][0]['content']['components'][0]['data']['meta']['limit'] 100
json_data['data']['answer'][0]['txt'][0]['content']['components'][0]['data']['meta']['page'] 1
json_data['data']['answer'][0]['txt'][0]['content']['components'][0]['data']['meta']['extra']['row_count'] 1364
    """
    _1 = json_data['data']['answer'][0]['txt'][0]['content']['components'][0]['data']
    _2 = _1['meta']

    datas = _1['datas']
    columns = _1['columns']
    page = _2['page']
    limit = _2['limit']
    row_count = _2['extra']['row_count']

    return datas, columns, page, limit, row_count


def getDataList(json_data):
    """
json_data['answer']['components'][0]['data']['datas']
json_data['answer']['components'][0]['data']['meta']['page']
json_data['answer']['components'][0]['data']['meta']['limit']
json_data['answer']['components'][0]['data']['meta']['extra']['row_count']
    """
    _1 = json_data['answer']['components'][0]['data']
    _2 = _1['meta']

    datas = _1['datas']
    columns = _1['columns']
    page = _2['page']
    limit = _2['limit']
    row_count = _2['extra']['row_count']

    return datas, columns, int(page), int(limit), row_count


async def on_response(response):
    if response.url == _PAGE1_:
        P.update(*get_robot_data(await response.json()))
    if response.url == _PAGE2_:
        P.update(*getDataList(await response.json()))


async def query(page: Page,
                w: str = "收盘价>1000元",
                type_: QueryType = 'stock',
                max_page: int = 5,
                rename: bool = False) -> pd.DataFrame:
    querytype = _querytype_.get(type_, None)
    assert querytype is not None, f"不支持的类型:{type_}"

    await page.route(re.compile(r'.*\.(?:jpg|jpeg|png|gif|webp)(?:$|\?)'), lambda route: route.abort())

    P.reset()
    # page.viewport_size # 取出来是None
    # 宽度<=768会认为是手机,>768是PC
    await page.set_viewport_size({"width": 1280, "height": 800})
    async with page.expect_response(_PAGE1_) as response_info:
        await page.goto(f"https://www.iwencai.com/unifiedwap/result?w={w}&querytype={querytype}", wait_until="load")
    await on_response(await response_info.value)

    while P.has_next(max_page):
        logger.info("当前页为:{}, 点击`下页`", P.current())
        async with page.expect_response(_PAGE2_) as response_info:
            await page.get_by_text("下页").click()
        await on_response(await response_info.value)

    return P.get_dataframe(rename)



from enum import Enum


class QueryType(Enum):
    """查询类型"""
    CNStock = 'A股'
    HKStock = '港股'
    USStock = '美股'
    Index = '指数'
    Fund = '基金'
    ETF = 'ETF'
    ConBond = '可转债'
    Board = '板块'
    Info = '资讯'


class Site(Enum):
    """站点"""
    EastMoney = '东方财富'  # 东方财富 条件选股
    TDX = '通达信'  # 通达信 问小达
    THS = '同花顺'  # 同花顺 问财


class Provider(Enum):
    """提供商"""
    Nami = '纳米搜索'  # 360 纳米搜索
    YuanBao = '腾讯元宝'  # 腾讯元宝
    BaiDu = '百度AI搜索'  # 百度AI搜索
    # YiYan = '文心一言'  # 百度文心一言



from ._version import __version__

from .enums import QueryType, Site, Provider
from .tool import BrowserManager, query, chat

TIMEOUT = 1000 * 60 * 3  # 3分钟，在抓取EventStream数据时等待数据返回，防止外层30秒超时
TIMEOUT_60 = 1000 * 60  # 1分钟

# TODO 临时测试
# TIMEOUT = None
# TIMEOUT_60 = None



from pathlib import Path
from typing import List, Tuple


def is_image(path: str) -> bool:
    """判断是否是图片文件"""
    img_ext = ['.jpg', '.jpeg', '.png', '.gif', '.bmp', '.webp']
    ext = Path(path).suffix.lower()
    return ext in img_ext


def split_images(files: List[str]) -> Tuple[List[str], List[str]]:
    """图片列表分成两部分"""
    imgs = []
    docs = []
    for f in files:
        if is_image(f):
            imgs.append(f)
        else:
            docs.append(f)
    return imgs, docs


class GlobalVars:
    """全局变量"""

    def __init__(self):
        self.text = ""

    def set_text(self, text):
        self.text = text

    def get_text(self):
        return self.text



"""
东方财富 条件选股
https://xuangu.eastmoney.com/

1. 部分数据中包含中文单位，如万亿等，导致无法转换为数字，如VOLUME
2. 东财翻页需要提前手工登录
3. 东财翻页是页面已经翻了，然后等数据来更新，懒加载
"""
import re

import pandas as pd
from loguru import logger
from playwright.async_api import Page

from mcp_query_table.enums import QueryType

# 查询结果
# 'https://np-pick-b.eastmoney.com/api/smart-tag/stock/v3/pw/search-code'
# 'https://np-pick-b.eastmoney.com/api/smart-tag/fund/v3/pw/search-code'
# 'https://np-pick-b.eastmoney.com/api/smart-tag/hk/v3/pw/search-code'
# 'https://np-pick-b.eastmoney.com/api/smart-tag/cb/v3/pw/search-code'
# 'https://np-pick-b.eastmoney.com/api/smart-tag/etf/v3/pw/search-code'
# 'https://np-pick-b.eastmoney.com/api/smart-tag/bkc/v3/pw/search-code'
# 'https://np-tjxg-b.eastmoney.com/api/smart-tag/bkc/v3/pw/search-code'
_PAGE1_ = 'https://*.eastmoney.com/api/smart-tag/*/v3/pw/search-code'

_type_ = {
    QueryType.CNStock: 'stock',
    QueryType.Fund: 'fund',
    QueryType.HKStock: 'hk',
    QueryType.ConBond: 'cb',
    QueryType.ETF: 'etf',
    QueryType.Board: 'bk',  # 比较坑，bkc和bkc的区别
}


def convert_type(type):
    if type == 'Double':
        return float
    if type == 'String':
        return str
    if type == 'Long':
        return int
    if type == 'Boolean':
        return bool
    if type == 'INT':  # TODO 好像未出现过
        return int
    return type


class Pagination:
    def __init__(self):
        self.datas = {}
        self.pageNo = 1
        self.pageSize = 100
        self.total = 1024
        self.columns = []
        self.datas = {}

    def reset(self):
        self.datas = {}

    def update(self, pageNo, pageSize, total, columns, dataList):
        self.pageNo = pageNo
        self.pageSize = pageSize
        self.total = total
        self.columns = columns
        self.datas[self.pageNo] = dataList

    def has_next(self, max_page):
        c1 = self.pageNo * self.pageSize < self.total
        c2 = self.pageNo < max_page
        return c1 & c2

    def current(self):
        return self.pageNo

    def get_list(self):
        datas = []
        for k, v in self.datas.items():
            datas.extend(v)
        return datas

    def get_dataframe(self, rename: bool):
        columns = {x['key']: x['title'] for x in self.columns}
        dtypes = {x['key']: convert_type(x['dataType']) for x in self.columns}

        df = pd.DataFrame(self.get_list())
        for k, v in dtypes.items():
            if k == 'SERIAL':
                df[k] = df[k].astype(int)
                continue
            if isinstance(v, str):
                logger.info("未识别的数据类型 {}:{}", k, v)
                continue
            try:
                df[k] = df[k].astype(v)
            except ValueError:
                logger.info("转换失败 {}:{}", k, v)

        if rename:
            return df.rename(columns=columns)
        else:
            return df


P = Pagination()


def search_code(json_data):
    total = json_data['data']['result']['total']
    columns = json_data['data']['result']['columns']
    dataList = json_data['data']['result']['dataList']
    return total, columns, dataList


async def on_response(response):
    post_data_json = response.request.post_data_json
    pageNo = post_data_json['pageNo']
    pageSize = post_data_json['pageSize']
    P.update(pageNo, pageSize, *search_code(await response.json()))


async def query(page: Page,
                q: str = "收盘价>100元",
                type_: QueryType = 'stock',
                max_page: int = 5,
                rename: bool = True) -> pd.DataFrame:
    type = _type_.get(type_, None)
    assert type is not None, f"不支持的类型:{type_}"

    await page.route(re.compile(r'.*\.(?:jpg|jpeg|png|gif|webp)(?:$|\?)'), lambda route: route.abort())

    P.reset()
    async with page.expect_response(_PAGE1_) as response_info:
        # 这里不用处理输入编码问题
        await page.goto(f"https://xuangu.eastmoney.com/Result?q={q}&type={type}", wait_until="load")
    await on_response(await response_info.value)

    while P.has_next(max_page):
        logger.info("当前页为:{}, 点击`下一页`", P.current())

        # 这种写法解决了懒加载问题
        async with page.expect_response(_PAGE1_) as response_info:
            await page.get_by_role("button", name="下一页").click()
        await on_response(await response_info.value)

    return P.get_dataframe(rename)
